@@ -51,9 +51,7 @@
  * deformation algorithm.
  */
 
-#ifdef HAVE_CONFIG_H
 #include "clutter-build-config.h"
-#endif
 
 #define CLUTTER_ENABLE_EXPERIMENTAL_API
 #include "clutter-deform-effect.h"
@@ -149,7 +147,7 @@ clutter_deform_effect_set_actor (ClutterActorMeta *meta,
       ClutterActor *old_actor = clutter_actor_meta_get_actor (meta);
 
       if (old_actor != NULL)
-        g_signal_handler_disconnect (old_actor, priv->allocation_id);
+        g_clear_signal_handler (&priv->allocation_id, old_actor);
 
       priv->allocation_id = 0;
     }
@@ -168,18 +166,20 @@ clutter_deform_effect_set_actor (ClutterActorMeta *meta,
 }
 
 static void
-clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
+clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect,
+                                    ClutterPaintContext    *paint_context)
 {
   ClutterDeformEffect *self= CLUTTER_DEFORM_EFFECT (effect);
   ClutterDeformEffectPrivate *priv = self->priv;
   CoglHandle material;
   CoglPipeline *pipeline;
   CoglDepthState depth_state;
-  CoglFramebuffer *fb = cogl_get_draw_framebuffer ();
+  CoglFramebuffer *fb =
+    clutter_paint_context_get_framebuffer (paint_context);
 
   if (priv->is_dirty)
     {
-      ClutterRect rect;
+      graphene_rect_t rect;
       gboolean mapped_buffer;
       CoglVertexP3T2C4 *verts;
       ClutterActor *actor;
@@ -195,8 +195,8 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
        */
       if (clutter_offscreen_effect_get_target_rect (effect, &rect))
         {
-          width = clutter_rect_get_width (&rect);
-          height = clutter_rect_get_height (&rect);
+          width = graphene_rect_get_width (&rect);
+          height = graphene_rect_get_height (&rect);
         }
       else
         clutter_actor_get_size (actor, &width, &height);
@@ -215,7 +215,7 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
       if (verts == NULL)
         {
           mapped_buffer = FALSE;
-          verts = malloc (sizeof (*verts) * priv->n_vertices);
+          verts = g_malloc (sizeof (*verts) * priv->n_vertices);
         }
       else
         mapped_buffer = TRUE;
@@ -272,7 +272,7 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
                                 0, /* offset */
                                 verts,
                                 sizeof (*verts) * priv->n_vertices);
-          free (verts);
+          g_free (verts);
         }
 
       priv->is_dirty = FALSE;
@@ -284,6 +284,7 @@ clutter_deform_effect_paint_target (ClutterOffscreenEffect *effect)
   /* enable depth testing */
   cogl_depth_state_init (&depth_state);
   cogl_depth_state_set_test_enabled (&depth_state, TRUE);
+  cogl_depth_state_set_test_function (&depth_state, COGL_DEPTH_TEST_FUNCTION_LEQUAL);
   cogl_pipeline_set_depth_state (pipeline, &depth_state, NULL);
 
   /* enable backface culling if we have a back material */
@@ -425,7 +426,7 @@ clutter_deform_effect_init_arrays (ClutterDeformEffect *self)
                               static_indices,
                               n_indices);
 
-  free (static_indices);
+  g_free (static_indices);
 
   priv->n_vertices = (priv->x_tiles + 1) * (priv->y_tiles + 1);
 
