@@ -126,9 +126,6 @@ get_property (ClutterInputDevice *device,
 
   if (rc == Success && type_ret == type && format_ret == format && nitems_ret >= nitems)
     {
-      if (nitems_ret > nitems)
-        g_warning ("Property '%s' for device '%s' returned %lu items, expected %lu",
-                   property, clutter_input_device_get_device_name (device), nitems_ret, nitems);
       return data_ret;
     }
 
