@@ -263,6 +263,10 @@ int  meta_display_get_primary_monitor  (MetaDisplay   *display);
 META_EXPORT
 int  meta_display_get_current_monitor  (MetaDisplay   *display);
 
+META_EXPORT
+const gchar* meta_display_get_monitor_name (MetaDisplay *display,
+                                            int          monitor);
+
 META_EXPORT
 void meta_display_get_monitor_geometry (MetaDisplay   *display,
                                         int            monitor,
@@ -357,6 +361,9 @@ void meta_display_set_desklets_above (MetaDisplay *display, gboolean above);
 META_EXPORT
 gboolean meta_display_get_desklets_above (MetaDisplay *display);
 
+META_EXPORT
+void meta_display_push_tile (MetaDisplay *display, MetaWindow *window, MetaMotionDirection direction);
+
 #endif
 
 
